/*!
 *  @file GEENYmodem.h
 *
 *
 *	MIT license (see license.txt)
 */

#ifndef _GEENYMODEM_H
#define _GEENYMODEM_H

#define APN "all2hub-tb"

#define MQTT_BROKER_URL "modems.tingg.io"
#define MQTT_BROKER_PORT 1883

#define MODEM_MANAGER_URL "modems.tingg.io"

#define MODEM_MANAGER_PORT 80

#define MODEM_MANAGER_POWERUP_RESOURCE "/api/v1/routinghub/powerup"
#define MODEM_MANAGER_WAKEUP_RESOURCE "/api/v1/routinghub/wakeup"

#define GPRS_USER ""
#define GPRS_PWD ""

#define TOPIC_INIT "init"

#define TINY_GSM_MODEM_SIM800

#include "Arduino.h"
#include <TinyGsmClient.h>
#include <PubSubClient.h>
#include <ArduinoJson.h>

/*!
 *    @brief  Class that stores state and functions for interacting with
 *            the GEENYmodem hardware
 */
class GEENYmodem
{
public:
  GEENYmodem();
  ~GEENYmodem();

  bool begin(Stream &port);
  bool begin(Stream &port, Stream &debugport);
  void reset(void);
  uint8_t getNetworkStatus(void);
  uint16_t getRSSI(void);
  String getModemInfo(void);
  bool isNetworkConnected(void);
  bool isMQTTConnected(void);
  String getSimCCID(void);
  String getIMEI(void);
  String getIMSI(void);

  void mqttCallback(char *topic, byte *payload, unsigned int len);
  boolean connectMQTT(void);
  void disconnectMQTT(void);
  boolean mqttPublish(const char *topic, char *payload);

  boolean connectGPRS(void);

  String getOperator(void);
  String getLocalIP(void);

  boolean powerup(void);

  // this method is only just for manufacturing for activating the modem in the tingg modem manager
  boolean wakeup(char *);

private:
  bool _init();
  TinyGsm *gprs;
  Stream *debug;

  TinyGsmClient *client;
  PubSubClient *mqtt;

  char thing_id[50] = {'\0'};
  char thing_key[50] = {'\0'};
  bool poweredup = false;

  const char *mqttBrokerUrl = MQTT_BROKER_URL;

  // buffer for sprintf
  char buffer[750];

  bool mqttstatus = false;
};

#endif